//Copyright (c) Dmitry Kramkov, 2000-2006. All rights reserved. 

#ifndef __SampleExam1_hpp__
#define __SampleExam1_hpp__

#include "cfl/Data.hpp"
#include "cfl/AssetModel.hpp"
#include "cfl/InterestRateModel.hpp"

/**
 * \file   SampleExam1.hpp
 * \author Dmitry Kramkov
 * \date   2000-2006
 * 
 * \brief Exam for the course Financial Computing with C++. 
 *
 */


//! Problems for the course.   
namespace prb
{
  /** 
   * \defgroup prbData Construction of data curves.
   * 
   * This module contains functions that construct input data curves,
   * such as volatility, forward, and discount, for financial models.
   */
  //@{

  /** 
   * Computes the curve of forward prices on a stock paying discrete 
   * dividends. 
   *
   * \param dSpot The spot price of the stock. 
   * \param rFixedDividendsTimes The vector of times, when fixed
   * dividends are paid. 
   * \param rFixedDividends The vector of fixed dividends (in
   * cash). 
   * \param rDiscount The discount curve. 
   * \param dInitialTime The initial time.
   * 
   * \return The forward price curve for the stock. 
   */
  cfl::Function 
  forwardStockDividends(double dSpot, 
			std::vector<double> & rFixedDividendsTimes, 
			std::vector<double> & rFixedDividends, 
			const cfl::Function & rDiscount, 
			double dInitialTime);

  //@}

  /**
   * \defgroup prbAssetStd Standard and barrier options on a single stock. 
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::AssetModel.
   */
  //@{

  /** 
   * Computes the value of the <strong>up-range-out put option</strong>. 
   * The option is canceled after first \a iOutTimes barrier times, 
   * when the stock price is above \a dUpperBarrier. Otherwise, 
   * it behaves as standard European put option with maturity \a dMaturity
   * and strike \a dStrike. 
   *
   * \param dUpperBarrier The upper barrier.  
   * \param iOutTimes The number of barrier events ("stock price is 
   * above \a dUpperBarrier") that cancels the option. 
   * \param rBarrierTimes The vector of barrier times. The first
   * time is greater than the initial time. The last barrier time is less 
   * than \a dMaturity. 
   * \param dStrike The strike of the put option. 
   * \param dMaturity The maturity of the option.  
   * \param rModel Reference to implementation of AssetModel. 
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.  
   */   
  cfl::MultiFunction upRangeOutPut(double dUpperBarrier, unsigned iOutTimes, 
				   const std::vector<double> & rBarrierTimes,
				   double dStrike, double dMaturity, 
				   cfl::AssetModel & rModel);

  //@}

  /**
   * \defgroup prbInterestRateStd Standard and barrier options on interest rates.
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::InterestRateModel.
   */
  //@{

  /** 
   * Computes the <strong>future price of  the cheapest bond to
   * deliver</strong>. 
   * The set of times, when the future price is evaluated is given by 
   * \code 
   * t(i) = rModel.initialTime() +
   * i*(dFutureMaturity-rModel.initialTime())/iFutureTimes, 
   * i = 0, ... , iFutureTimes-1
   * \endcode
   * Denote by <em>F(t(i))</em> the future price determined at time
   * <em>t(i)</em>. 
   * Recall that it costs nothing to enter into the future contract
   * at time <em>t(i)</em> and that short position entered at time
   * <em>t(i)</em> assumes the following transactions:
   * <ol>
   * <li> At time <em>t(k)</em> such that <em>i < k < iFutureTimes
   * </em> short position pays the amount <em>F(t(k+1)) -
   * F(t(k))</em>.  
   * <li> At maturity \a dFutureMaturity short position receives future
   * price 
   * <em>F(t(iFutureTimes-1))</em> defined at the previous time step
   * and delivers any bond from \a rBonds at its discretion.  
   * </ol> 
   * 
   * \param dFutureMaturity The maturity of the future contract. 
   * \param iFutureTimes The number of future times, that is, the
   * times when the future price is determined. 
   * \param rBonds The parameters of the underlying bonds. We assume
   * that the issue times for all bonds equal \a dFutureMaturity. 
   * \param rModel Reference to implementation of InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction 
  futureOnCheapToDeliver(double dFutureMaturity, 
			 unsigned iFutureTimes, 
			 const std::vector<cfl::Data::CashFlow> & rBonds,
			 cfl::InterestRateModel & rModel);

  //@}

  /** 
   * \defgroup prbPath Path dependent options.  
   *
   * This module contains functions that compute prices of path
   * dependent options in cfl::AssetModel and cfl::InterestRateModel.
   */
  //@{

  /** 
   * Computes the price of the <strong>clique option</strong>. The payoff 
   * of the option at maturity
   * is given by:
   * \f[
   * V(T) = \frac1{N} \sum_{i=1}^N \max(S(t_i) - K(t_i), 0),
   * \f]
   * where the summation is done over the set of averaging times 
   * \a rAverageTimes and \f$S(t_i)\f$ and \f$K(t_i)\f$ are, respectively, 
   * the spot price and the strike at \f$t_i\f$. The value of the strike 
   * \f$K(t)\f$ at time \f$t\f$ is given by
   * the initial strike \a dInitialStrike, if there are no reset times
   * before \f$t\f$, and the spot price at the previous reset time,
   * otherwise. 
   *
   * \param dMaturity The maturity of the option. 
   * \param rAverageTimes The vector of averaging times. The last averaging
   * time is less than the maturity. 
   * \param rResetTimes The vector of reset times. Neither of reset times
   * is an exercise time. 
   * The last reset time is less than the last exercise time. 
   * \param dInitialStrike The initial strike. 
   * \param rModel Reference to implementation of AssetModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction clique(double dMaturity, 
			    const std::vector<double> & rAverageTimes, 
			    const std::vector<double> & rResetTimes, 
			    double dInitialStrike,  
			    cfl::AssetModel & rModel);
    
  //@}
}

#endif // of __SampleExam1_hpp__
