#include <iostream>
#include <fstream>
#include "test/Main.hpp"
#include "test/Black.hpp"
#include "test/HullWhite.hpp"
#include "test/Data.hpp"
#include "SampleExam1/test_SampleExam1.hpp"
#include "SampleExam1/SampleExam1.hpp"

using namespace std;
using namespace prb;
using namespace cfl;
using namespace cfl::test;
using namespace cfl::test::Black;
using namespace cfl::test::HullWhite;

namespace test_prb
{
  static const char * c_pDataFile = 
  "../SampleExam1/Output/SampleExam1Data.txt";
  static const char * c_pDataProject = "TEST OF DATA CURVES";
  static const char * c_pBlackStandardFile = 
  "../SampleExam1/Output/SampleExam1BlackStandard.txt";
  static const char * c_pBlackStandardProject = 
  "TEST OF STANDARD OPTIONS IN BLACK MODEL FOR SINGLE ASSET";
  static const char * c_pHullWhiteStandardFile = 
  "../SampleExam1/Output/SampleExam1HullWhiteStandard.txt";
  static const char * c_pHullWhiteStandardProject = 
  "TEST OF STANDARD OPTIONS IN HULL AND WHITE  MODEL FOR INTEREST RATES";
  static const char * c_pBlackPathDependentFile = 
  "../SampleExam1/Output/SampleExam1BlackPathDependent.txt";
  static const char * c_pBlackPathDependentProject = 
  "TEST OF PATH DEPENDENT OPTIONS IN BLACK MODEL FOR SINGLE ASSET";

  void data();
  void standard(const cfl::Black::Data & rData);
  void pathDependent(const cfl::Black::Data & rData);
  void standard(const cfl::HullWhite::Data & rData);
}

int main()
{
  using namespace cfl::test;
  using namespace std;
  try{
    cfl::test::Data::project(test_prb::data, test_prb::c_pDataProject, 
			     test_prb::c_pDataFile);
    cfl::test::pause();
    cfl::test::Black::project(test_prb::standard, 
			      test_prb::c_pBlackStandardProject, 
			      test_prb::c_pBlackStandardFile);
    cfl::test::pause();
    cfl::test::Black::project(test_prb::pathDependent, 
			      test_prb::c_pBlackPathDependentProject, 
			      test_prb::c_pBlackPathDependentFile);
    cfl::test::pause();
    cfl::test::HullWhite::project(test_prb::standard, 
				  test_prb::c_pHullWhiteStandardProject, 
				  test_prb::c_pHullWhiteStandardFile);
    cfl::test::pause();
  }
  catch(const std::exception & rError) {
    std::cerr<< rError.what() << endl;
    cfl::test::pause();
  }
}

void test_prb::data()
{
  prb::test::forwardStockDividends();
}

void test_prb::standard(const cfl::Black::Data & rData)
{
  double dQuality = cfl::test::Black::c_dQuality;
  cout << "quality = " << dQuality << endl;
  double dSpot = rData.forward()(rData.initialTime());
  double dInterval = cfl::test::Black::c_dInterval;
  unsigned iPoints = cfl::test::Black::c_iPoints;

  AssetModel uModel = cfl::Black::model(rData, dInterval, dQuality);

  using namespace prb::test; 
  
  report(upRangeOutPut, dSpot, dInterval, iPoints, uModel);
}


void test_prb::pathDependent(const cfl::Black::Data & rData)
{
  double dQuality = cfl::test::Black::c_dQuality;
  cout << "quality = " << dQuality << endl;
  double dPathDependQuality = cfl::test::Black::c_dPathDependQuality;
  cout << "quality for additional state process = " 
       << dPathDependQuality << endl;
  double dSpot = rData.forward()(rData.initialTime());
  double dInterval = cfl::test::Black::c_dInterval;
  unsigned iPoints = cfl::test::Black::c_iPoints;

  AssetModel uModel = cfl::Black::model(rData, dInterval, 
					dQuality, dPathDependQuality);

  using namespace prb::test;

  report(clique, dSpot, dInterval, iPoints, uModel);
}

void test_prb::standard(const cfl::HullWhite::Data & rData)
{
  double dQuality = cfl::test::HullWhite::c_dQuality;
  cout << "quality = " << dQuality << endl;
  const double c_dDay = 1./365.;
  double dRate = 
    - std::log(rData.discount()(rData.initialTime()+c_dDay))/c_dDay;
  double dInterval = cfl::test::HullWhite::c_dInterval;
  unsigned iPoints = cfl::test::HullWhite::c_iPoints;

  InterestRateModel uModel = cfl::HullWhite::model(rData, dInterval, dQuality);

  using namespace prb::test;

  report(futureOnCheapToDeliver, dRate, dInterval, iPoints, uModel);
}

void prb::test::forwardStockDividends() 
{
  cfl::test::print("FORWARD PRICES FOR A STOCK WITH FIXED DIVIDENDS");
  cout << endl;
  double dSpot = 1000;
  double dRate = 0.12;
  double dInitialTime = 1.;
  Function uDiscount = cfl::Data::discount(dRate, dInitialTime);
  unsigned iTimes = 10;
  std::vector<double> uTimes(iTimes);
  double dPeriod = 0.5;
  uTimes.front() = dInitialTime + dPeriod;
  std::transform(uTimes.begin(), uTimes.end()-1, uTimes.begin()+1, 
		 [dPeriod](double dX){ return dX+dPeriod; });
  double dDividend = 10.;
  std::vector<double> uDividends(iTimes, dDividend);

  cout << "initial time = " << dInitialTime << endl;
  cout << "interest rate = " << dRate << endl;
  cout << "spot = " << dSpot << endl;
  cout << "dividend times: " << endl;
  cfl::test::print(uTimes.begin(), uTimes.end());
  cout << "dividends: " << endl;
  cfl::test::print(uDividends.begin(), uDividends.end());
  

  Function uForwardStockDividends = 
    prb::forwardStockDividends(dSpot, uTimes, uDividends, 
			       uDiscount, dInitialTime); 
  double dInterval = 2.33;
  cout << "output forward prices: " << endl;
  cfl::test::Data::print(uForwardStockDividends, dInitialTime, dInterval); 
}


MultiFunction prb::test::upRangeOutPut(AssetModel & rModel) 
{
  cfl::test::print("UP-RANGE-OUT PUT OPTION IN ASSET MODEL");
  cout << endl;
  double dStrike =cfl::test::Black::c_dStrike;
  double dMaturity =cfl::test::Black::c_dMaturity;
  double dUpperBarrier =cfl::test::Black::c_dSpot*1.02;
  const std::vector<double> uBarrierTimes =cfl::test::Black::c_uBarrierTimes;
  unsigned iOutTimes = std::min<int>(uBarrierTimes.size(),4);
  cout << "strike = " << dStrike << endl;
  cout << "maturity = " << dMaturity << endl;
  cout << "upper barrier = " << dUpperBarrier << endl;
  cout << "barrier times = "<< endl;
  cfl::test::print(uBarrierTimes.begin(), uBarrierTimes.end());
  cout << "number of barrier events to cancel put = " << iOutTimes;
	
  return prb::upRangeOutPut(dUpperBarrier, iOutTimes, 
			    uBarrierTimes, dStrike, dMaturity, 
			    rModel);
}


cfl::MultiFunction 
prb::test::futureOnCheapToDeliver(InterestRateModel & rModel)
{
  cfl::test::print("FUTURE PRICE OF CHEAPEST TO DELIVER COUPON BOND IN INTEREST RATE MODEL");
  cout << endl;
  double dFutureMaturity = 0.5;
  unsigned iNumberOfFutureTimes = 20;
  cout << "maturity of future contract = " << dFutureMaturity << endl;
  cout << "number of future times = " << iNumberOfFutureTimes << endl;
  unsigned iNumberOfBonds = 5;
  std::vector<cfl::Data::CashFlow> uBonds(iNumberOfBonds);
  double dNotional = cfl::test::HullWhite::c_dNotional;
  double dRate = cfl::test::HullWhite::c_dYield;
  double dPeriod = 0.25;
  unsigned iNumberOfPeriods = 4; 
  for (unsigned i=0; i<iNumberOfBonds; i++) {
    cout << "Bond " << i << ":" << endl;
    uBonds[i].notional = dNotional;
    cout << "notional = " << uBonds[i].notional << endl;
    uBonds[i].rate = dRate;
    cout << "coupon rate = " << uBonds[i].rate << endl;
    uBonds[i].period = dPeriod;
    cout << "period = " << uBonds[i].period << endl;
    uBonds[i].numberOfPayments = iNumberOfPeriods + i; 
    cout << "number of payments = " << uBonds[i].numberOfPayments << endl;
  }
  return prb::
    futureOnCheapToDeliver(dFutureMaturity, iNumberOfFutureTimes, 
			   uBonds, rModel);
}

MultiFunction prb::test::clique(cfl::AssetModel & rModel)
{
  cfl::test::print("CLIQUE OPTION IN ASSET MODEL");
  cout << endl;
  double dStrike =cfl::test::Black::c_dStrike;
  const std::vector<double> uAverageTimes =cfl::test::Black::c_uExerciseTimes;
  const std::vector<double> uResetTimes =cfl::test::Black::c_uBarrierTimes;
  double dMaturity = uAverageTimes.back() + 5./256.; //one week later

  cout << "initial strike = " << dStrike << endl;
  cout << "maturity = " << dMaturity << endl;
  cout << "averaging times = "<< endl;
  cfl::test::print(uAverageTimes.begin(), uAverageTimes.end());
  cout << "reset times = "<< endl;
  cfl::test::print(uResetTimes.begin(), uResetTimes.end());

  return prb::
    clique(dMaturity, uAverageTimes, uResetTimes, dStrike, rModel);
}
