//Copyright (c) Dmitry Kramkov, 2000-2006. All rights reserved. 

#ifndef __SampleExam2_hpp__
#define __SampleExam2_hpp__

#include "cfl/Data.hpp"
#include "cfl/AssetModel.hpp"
#include "cfl/InterestRateModel.hpp"

/**
 * \file   SampleExam2.hpp
 * \author Dmitry Kramkov
 * \date   2000-2006
 * 
 * \brief Exam for the course Financial Computing with C++. 
 *
 */


//! Problems for the course.   
namespace prb
{
  /** 
   * \defgroup prbData Construction of data curves.
   * 
   * This module contains functions that construct input data curves,
   * such as volatility, forward, and discount, for financial models.
   */
  //@{

  /** 
   * Computes discount curve from a family of swap rates. 
   * The computation is done in two steps. First, we deduce discount 
   * factors for the maturities of swap contracts. Second, we compute 
   * discount curve using log-linear interpolation of discount 
   * factors obtained in the first step.  
   * 
   * \param rSwapRates The vector of known swap rates. The maturity
   * of the first swap is given by
   * <em>dInitialTime+dPeriod</em>. Two adjacent swap contracts are
   * separated by the time interval \a dPeriod. 
   * \param dPeriod The period between two payments. This is also
   * the time interval between maturities of the adjacent swap
   * contracts. 
   * \param dInitialTime The initial time. 
   * 
   * \return The discount curve obtained from known swap rates
   * by log-linear interpolation of discount factors. 
   */
  cfl::Function 
  discountSwapLogLinearInterp(const std::vector<double> & rSwapRates, 
			      double dPeriod, double dInitialTime);

  //@}

  /**
   * \defgroup prbAssetStd Standard and barrier options on a single stock. 
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::AssetModel.
   */
  //@{

  /** 
   * Computes the value of the <strong>"BOOST" (Banking On Overall
   * Stability) option</strong>. This contract terminates at the first 
   * barrier time when the stock price hits either of the barriers. 
   * At the exit time the holder of the option receives the payoff given 
   * by the product of the notional amount \a dNotional on the percentage
   * of the barrier times that the price of the stock spends inside
   * of two barriers: 
   * \code 
   * Payoff at exit time = Notional * (Number of barrier times strictly 
   * smaller then the exit time)/(Total number of barrier times)
   * \endcode
   * If the price of the stock never exits the
   * barriers, then at last barrier time the holder of the
   * option is paid the notional amount \a dNotional. 
   * 
   * \param dNotional The notional amount.  
   * \param dLowerBarrier The lower barrier.  
   * \param dUpperBarrier The upper barrier.  
   * \param rBarrierTimes The vector of barrier times. The first
   * time is greater than the initial time. 
   * \param rModel Reference to implementation of AssetModel. 
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.  
   */
  cfl::MultiFunction boost(double dNotional, double dLowerBarrier, 
			   double dUpperBarrier,  
			   const std::vector<double> & rBarrierTimes,
			   cfl::AssetModel & rModel);

  //@}

  /**
   * \defgroup prbInterestRateStd Standard and barrier options on interest rates.
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::InterestRateModel.
   */
  //@{

  /** 
   * Computes the price of <strong>puttable bond with resettable
   * coupon</strong> at its issue time.  
   * In this contract, after coupon payment the issuer can reset
   * the coupon rate from the original (higher) value
   * <em>rBond.rate</em> to the (lower) reset value \a
   * dResetCouponRate. After that at any payment time greater or
   * equal the reset time and less than maturity the holder can sell
   * the bond back to the issuer for the redemption value
   * <em>dRedemptionPrice*rBond.notional</em>. 
   * 
   * \param rBond The parameters of the underlying coupon bond. The
   * initial coupon rate is given by <em>rBond.rate</em>. 
   * \param dResetCouponRate The coupon rate after reset. Usually,
   * this rate is smaller than the original coupon rate. 
   * \param dRedemptionPrice The redemption price of the bond as percentage
   * of the notional.  Typically, <em>dRedemptionPrice<1</em>. 
   * \param rModel Reference to implementation of InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction resetCouponPutBond(const cfl::Data::CashFlow & rBond,  
					double dResetCouponRate,
					double dRedemptionPrice,
					cfl::InterestRateModel & rModel);

  //@}

  /** 
   * \defgroup prbPath Path dependent options.  
   *
   * This module contains functions that compute prices of path
   * dependent options in cfl::AssetModel and cfl::InterestRateModel.
   */
  //@{

  /** 
   * Computes the price of <strong>resettable interest rate
   * cap</strong>.
   * In this contract, the cap rate is reset at times \a rResetTimes
   * to the float rate plus spread \a dSpread. 
   * 
   * \param rCap The parameters of interest rate cap. The initial
   * cap rate is given by <em>rCap.rate</em>. 
   * \param dSpread The spread added to float rate in order to
   * compute resettable cap rate. 
   * \param rResetTimes The vector of reset times for cap rate. 
   * \param rModel Reference to implementation of InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction resetCap(const cfl::Data::CashFlow & rCap, 
			      double dSpread, 
			      const std::vector<double> & rResetTimes,
			      cfl::InterestRateModel & rModel); 

  //@}
}

#endif // of __SampleExam2_hpp__
