#include "SampleExam2/SampleExam2.hpp" 

using namespace cfl;

Slice rate(unsigned iTime, double dPeriod, 
	   const cfl::InterestRateModel & rModel)
{
  PRECONDITION(iTime < rModel.eventTimes().size());
  
  double dTime = rModel.eventTimes()[iTime] + dPeriod;
  Slice uDiscount = rModel.discount(iTime, dTime);
  return (1./uDiscount - 1.)/dPeriod;
}

class HistRate: public IResetValues
{
public:
  HistRate(double dPeriod, const InterestRateModel & rModel) 
    :m_dPeriod(dPeriod), m_rModel(rModel)
  {}
    
  Slice resetValues(unsigned iTime, double dBeforeReset) const 
  {
    return rate(iTime, m_dPeriod, m_rModel);
  }
    
private:
  double m_dPeriod;
  const InterestRateModel & m_rModel;
};

cfl::PathDependent 
histRate(double dPeriod, double dInitialRate, 
	 const std::vector<unsigned> & rResetTimes,  
	 const cfl::InterestRateModel & rModel)
{
  return PathDependent(new HistRate(dPeriod, rModel), 
		       rResetTimes, dInitialRate);
}

cfl::MultiFunction prb::
resetCap(const Data::CashFlow & rCap, double dSpread, 
	 const std::vector<double> & rResetTimes, InterestRateModel & rModel)
{
  PRECONDITION(rResetTimes.front() > rModel.initialTime());

  //cap times include initial time plus all payment times except the last one
  std::vector<double> uCapTimes(rCap.numberOfPayments, rModel.initialTime());
  std::transform(uCapTimes.begin(), uCapTimes.end()-1, uCapTimes.begin()+1, 
		 [&rCap](double dX){ return dX+rCap.period; }); 

  ASSERT(uCapTimes.back() >= rResetTimes.back());

  std::vector<double> uEventTimes(uCapTimes.size()+rResetTimes.size());
  std::vector<double>::iterator itEnd = 
    std::set_union(uCapTimes.begin(), uCapTimes.end(), 
		   rResetTimes.begin(), rResetTimes.end(), 
		   uEventTimes.begin());
  uEventTimes.resize(itEnd - uEventTimes.begin());
  rModel.assignEventTimes(uEventTimes);
	
  std::vector<unsigned> uResetIndexes(rResetTimes.size());
  for (unsigned iI=0; iI<uResetIndexes.size(); iI++) {
    uResetIndexes[iI] = 
      std::lower_bound(uEventTimes.begin(), uEventTimes.end(), 
		       rResetTimes[iI]) - uEventTimes.begin();
  }
  unsigned iState = 
    rModel.addState(histRate(rCap.period, rCap.rate-dSpread, uResetIndexes, 
			     rModel));
  //last minus one payment time
  int iTime = uEventTimes.size()-1;
  double dTime = rModel.eventTimes()[iTime];
  ASSERT(std::binary_search(uCapTimes.begin(),uCapTimes.end(),dTime));
  Slice uDiscount = rModel.discount(iTime, dTime + rCap.period);
  //current value of next float payment
  Slice uFloat = 1. - uDiscount;
  //current value of next fixed payment
  Slice uFixed = 
    uDiscount * (rModel.state(iTime, iState) + dSpread)*rCap.period;
  Slice uCap = max(uFloat-uFixed, 0);

  while (iTime > 0) {
    //uCap is the value of future payments
    iTime--;
    uCap.rollback(iTime);
    dTime = rModel.eventTimes()[iTime];
    if (std::binary_search(uCapTimes.begin(), uCapTimes.end(), dTime)) {
      //the current time is a cap time
      uDiscount = rModel.discount(iTime,dTime+rCap.period);
      uFloat = 1.-uDiscount;
      uFixed = uDiscount*(rModel.state(iTime, iState)+dSpread)*rCap.period;
      uCap += max(uFloat-uFixed, 0);
    }		
  }
  uCap *= rCap.notional;
  
  return interpolate(uCap, iState);	
}
