//Copyright (c) Dmitry Kramkov, 2000-2008. All rights reserved. 

#ifndef __SampleExam3_hpp__
#define __SampleExam3_hpp__

#include "cfl/Data.hpp"
#include "cfl/AssetModel.hpp"
#include "cfl/InterestRateModel.hpp"

/**
 * \file   SampleExam3.hpp
 * \author Dmitry Kramkov
 * \date   2008
 * 
 * \brief Final Exam for the course Financial Computing with C++. 
 *
 */


//! Problems for the course.   
namespace prb
{
  /** 
   * \defgroup prbData Construction of data curves.
   * 
   * This module contains functions that construct input data curves,
   * such as volatility, forward, and discount, for financial models.
   */
  //@{

  /** 
   * Computes the volatility curve \f$V = V(t)\f$ by applying the linear 
   * interpolation to 
   * the variance curve \f$D = D(t)\f$. The relation between the two curves 
   * is 
   * \f[
   *      D(t) = (t - t_0) V^2(t),
   * \f]
   *  where \f$t_0\f$ (= \a dInitialTime) is the initial time.  Note that 
   * the resulting 
   * volatility curve will be constant on the interval 
   * <code>[dInitialTime, rMaturities.front()]</code>
   * and given by <code>rVolatilities.front()</code>. 
   * 
   * \param rMaturities The maturities of known volatilities. We assume that 
   * <code> rMaturities.front() > dInitialTime </code>. 
   * \param rVolatilities The vector of known volatilities. 
   * \param dInitialTime The initial time. 
   * 
   * \return The volatility curve on the interval 
   * <code> [dInitialTime, rMaturities.back()] </code>. 
   */
  cfl::Function 
  volatilityLinearInterpOfVar(const std::vector<double> & rMaturities, 
			      const std::vector<double> & rVolatilities, 
			      double dInitialTime);

  //@}

  /**
   * \defgroup prbAssetStd Standard and barrier options on a single stock. 
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::AssetModel.
   */
  //@{

  /** 
   * At maturity \a dMaturity a holder can buy the American put option 
   * (with strike \a dPutStrike and exercise times \a rExerciseTimes) at 
   * the price \a dCompoundCallStrike. 
   * 
   * \param dMaturity The maturity of call option on American put.
   * \param dCompoundCallStrike The strike of call option on American put.
   * \param dPutStrike The strike of American put.
   * \param rExerciseTimes The exercise times for American put. The first 
   * exercise time \a rExerciseTimes.front() is strictly greater than 
   * \a dMaturity.
   * \param rModel Reference to implementation of AssetModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction  
  compoundCallAmericanPut(double dMaturity,
			  double dCompoundCallStrike, 
			  double dPutStrike, 
			  const std::vector<double> & rExerciseTimes, 
			  cfl::AssetModel & rModel);

  //@}

  /**
   * \defgroup prbInterestRateStd Standard and barrier options on interest rates.
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::InterestRateModel.
   */
  //@{

  /** 
   * Computes the price of <strong>callable capped floater</strong>. 
   * In this contract, at an exercise time \f$t_i\f$ the holder of the
   * option receives the coupon 
   * \f[
   *    N \delta t \times \min(L(t_{i-1},t_i)+ \delta L, C)
   * \f]
   * where \f$L(s,t)\f$ is the LIBOR rate computed at \f$s\f$ for maturity 
   * \f$t\f$, \f$N\f$  is the notional amount (\a rCap.notional), 
   * \f$\delta L\f$ is the spread over LIBOR (\a dLiborSpread), 
   * \f$C\f$ is the cap rate (\a rCap.rate) and 
   * \f$\delta t = t_{i} - t_{i-1}\f$ 
   * is the time interval between two payments (\a rCap.period). 
   * The seller can terminate the contract at any exercise time. 
   * In this case, in addition to the above coupon he pays the
   * notional. If the contract has not been terminated before, 
   * then at maturity the holder receives the above coupon plus notional.
   * 
   * \param rCap The parameters of the contract. Here <em>rCap.rate</em>
   *  defines the cap rate. 
   * \param dLiborSpread The spread over LIBOR rate. 
   * \param rModel Reference to implementation of InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction callableCappedFloater(const cfl::Data::CashFlow & rCap, 
					   double dLiborSpread, 
					   cfl::InterestRateModel & rModel);


  //@}

  /** 
   * \defgroup prbPath Path dependent options.  
   *
   * This module contains functions that compute prices of path
   * dependent options in cfl::AssetModel and cfl::InterestRateModel.
   */
  //@{
    
  /** 
   * Computes the price of <strong>index amortizing swap</strong>. 
   * In this contract, the sides make interest payments  similar 
   * to standard interest rate swap. However, the notional amount 
   * is reset at payment times by float (LIBOR) rate according to 
   * amortizing function \a fAmortizing. In addition, the swap is canceled
   * as soon as the remaining notional amount reaches the level below
   * \a dLowerThreshold percents of the original notional amount
   * \a rSwap.notional.
   * 
   * \param rSwap The parameters of the swap contract. 
   * \param fAmortizing The amortizing function for the notional (usually,
   * decreasing and taking values in (0,1))
   * \param dLowerThreshold The threshold for the cleanup call. 
   * \param rModel Reference to implementation of InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction indexAmortizingSwap(const cfl::Data::Swap & rSwap,
					 double (*fAmortizing)(double), 
					 double dLowerThreshold, 
					 cfl::InterestRateModel & rModel);

  //@}
}

#endif // of __SampleExam3_hpp__
