#include "SampleExam3/SampleExam3.hpp"

using namespace cfl;

Slice rate(unsigned iTime, double dPeriod, 
	   const cfl::InterestRateModel & rModel)
{
  PRECONDITION(iTime < rModel.eventTimes().size());
  PRECONDITION(dPeriod>cfl::c_dEps);   

  double dTime = rModel.eventTimes()[iTime] + dPeriod;
  Slice uDiscount = rModel.discount(iTime, dTime);
  return (1./uDiscount - 1.)/dPeriod;
}

class AmortizingNotional: public cfl::IResetValues
{
public: 
  AmortizingNotional(double (*fAm)(double), double dPeriod, 
		     const cfl::InterestRateModel & rModel)
    :m_rModel(rModel), m_fAm(fAm), m_dPeriod(dPeriod)
  {}
  Slice resetValues(unsigned iTime, double dBeforeReset) const {
    Slice uRate = rate(iTime,m_dPeriod,m_rModel);
    Slice uAm  = uRate.apply(m_fAm); 
    return dBeforeReset*uAm; 
  }
private:
  const cfl::InterestRateModel & m_rModel; 
  double (*m_fAm)(double); 
  double m_dPeriod; 
};

cfl::PathDependent 
amortizingNotional(double (*fAm)(double), double dPeriod, 
		   const std::vector<unsigned> & rIndexes, 
		   const InterestRateModel & rModel) 
{
  double dOrigin = 1.; 
  return PathDependent(new AmortizingNotional(fAm, dPeriod, rModel), 
		       rIndexes, dOrigin); 
}

cfl::MultiFunction prb::
indexAmortizingSwap(const Data::Swap & rSwap, double (*fAmortizing)(double), 
		    double dLowerThreshold, InterestRateModel & rModel)
{
  std::vector<double> 
    uEventTimes(rSwap.numberOfPayments, rModel.initialTime());
  std::transform(uEventTimes.begin(), uEventTimes.end()-1, 
		 uEventTimes.begin()+1, 
		 [&rSwap](double dX){ return dX+rSwap.period; }); 
  rModel.assignEventTimes(uEventTimes);
  
  std::vector<unsigned> uIndexes(uEventTimes.size()-1, 1);
  std::transform(uIndexes.begin(), uIndexes.end()-1, uIndexes.begin()+1, 
		 [](unsigned iX){ return iX+1; }); 
  int iNotional = 
    rModel.addState(amortizingNotional(fAmortizing, rSwap.period,
				       uIndexes,rModel));

  //last payment time before maturity
  int iTime = uEventTimes.size()-1;
  Slice uSwap = rModel.cash(iTime, 0.);
  double dFixedFactor = 1.+rSwap.rate * rSwap.period; 
  //percentage of amortizing notional
  Slice uNotional = rModel.state(iTime, iNotional);
  Slice uDiscount = 
    rModel.discount(iTime, rModel.eventTimes()[iTime] + rSwap.period); 
  //We pay fixed and receive float. We multiply on notional at the end.
  uSwap += uNotional * (1. - uDiscount*dFixedFactor); 

  while (iTime >0) { 
    //uSwap is the value of future payments if we continue.
    //We stop if notional is below threshold.
    uSwap *= indicator(uNotional, dLowerThreshold);
    iTime--; 
    uSwap.rollback(iTime);
    uNotional = rModel.state(iTime, iNotional);
    uDiscount = 
      rModel.discount(iTime, rModel.eventTimes()[iTime] + rSwap.period); 
    //We pay fixed. Add the value of next payment. 
    uSwap += uNotional * (1. - uDiscount*dFixedFactor); 
  }

  if (rSwap.payFloat) {
    uSwap *= -1.;
  }
  uSwap *= rSwap.notional;
   
  return interpolate(uSwap, iNotional);
}
