#include <iostream>
#include <fstream>
#include "test/Main.hpp"
#include "test/Black.hpp"
#include "test/HullWhite.hpp"
#include "test/Data.hpp"
#include "SampleExam3/test_SampleExam3.hpp"
#include "SampleExam3/SampleExam3.hpp"

using namespace std;
using namespace prb;
using namespace cfl;
using namespace cfl::test;
using namespace cfl::test::Black;
using namespace cfl::test::HullWhite;

namespace test_prb
{
  static const char * c_pDataFile = 
  "../SampleExam3/Output/SampleExam3Data.txt";
  static const char * c_pDataProject = "TEST OF DATA CURVES";
  static const char * c_pBlackStandardFile = 
  "../SampleExam3/Output/SampleExam3BlackStandard.txt";
  static const char * c_pBlackStandardProject = 
  "TEST OF STANDARD OPTIONS IN BLACK MODEL FOR SINGLE ASSET";
  static const char * c_pHullWhiteStandardFile = 
  "../SampleExam3/Output/SampleExam3HullWhiteStandard.txt";
  static const char * c_pHullWhiteStandardProject = 
  "TEST OF STANDARD OPTIONS IN HULL AND WHITE  MODEL FOR INTEREST RATES";
  static const char * c_pHullWhitePathDependentFile = 
  "../SampleExam3/Output/SampleExam3HullWhitePathDependent.txt";
  static const char * c_pHullWhitePathDependentProject = 
  "TEST OF PATH DEPENDENT OPTIONS IN HULL AND WHITE MODEL FOR INTEREST RATES";

  void data();
  void standard(const cfl::Black::Data & rData);
  void standard(const cfl::HullWhite::Data & rData);
  void pathDependent(const cfl::HullWhite::Data & rData);
}

int main()
{
  using namespace cfl::test;
  using namespace std;
  try{
    cfl::test::Data::project(test_prb::data, test_prb::c_pDataProject, 
			     test_prb::c_pDataFile);
    cfl::test::pause();
    cfl::test::Black::project(test_prb::standard, 
			      test_prb::c_pBlackStandardProject, 
			      test_prb::c_pBlackStandardFile);

    cfl::test::pause();
    cfl::test::HullWhite::project(test_prb::standard, 
				  test_prb::c_pHullWhiteStandardProject, 
				  test_prb::c_pHullWhiteStandardFile);
    cfl::test::pause();
    cfl::test::HullWhite::project(test_prb::pathDependent, 
				  test_prb::c_pHullWhitePathDependentProject, 
				  test_prb::c_pHullWhitePathDependentFile);
    cfl::test::pause();
  }
  catch(const std::exception & rError) {
    std::cerr<< rError.what() << endl;
    cfl::test::pause();
  }
}

void test_prb::data()
{
  prb::test::volatilityLinearInterpOfVar();
}

void test_prb::standard(const cfl::Black::Data & rData)
{
  double dQuality = cfl::test::Black::c_dQuality;
  cout << "quality = " << dQuality << endl;
  double dSpot = rData.forward()(rData.initialTime());
  double dInterval = cfl::test::Black::c_dInterval;
  unsigned iPoints = cfl::test::Black::c_iPoints;

  AssetModel uModel = cfl::Black::model(rData, dInterval, dQuality);

  report(prb::test::compoundCallAmericanPut, dSpot, dInterval, 
	 iPoints, uModel);
}


void test_prb::standard(const cfl::HullWhite::Data & rData)
{
  double dQuality = cfl::test::HullWhite::c_dQuality;
  cout << "quality = " << dQuality << endl;
  const double c_dDay = 1./365.;
  double dRate = 
    - std::log(rData.discount()(rData.initialTime()+c_dDay))/c_dDay;
  double dInterval = cfl::test::HullWhite::c_dInterval;
  unsigned iPoints = cfl::test::HullWhite::c_iPoints;

  InterestRateModel uModel = cfl::HullWhite::model(rData, dInterval, dQuality);

  report(prb::test::callableCappedFloater, dRate, dInterval, 
	 iPoints, uModel);
}

void test_prb::pathDependent(const cfl::HullWhite::Data & rData)
{
  double dQuality = cfl::test::HullWhite::c_dQuality;
  double dPathDependQuality =
    cfl::test::HullWhite::c_dPathDependQuality;
  
  cout << "quality = " << dQuality << endl;
  cout << "quality for additional state processes = " <<
    dPathDependQuality << endl;
  const double c_dDay = 1./365.;
  double dRate = 
    - std::log(rData.discount()(rData.initialTime()+c_dDay))/c_dDay;
  double dInterval = cfl::test::HullWhite::c_dInterval;
  unsigned iPoints = cfl::test::HullWhite::c_iPoints;

  InterestRateModel uModel =
    cfl::HullWhite::model(rData, dInterval, dQuality, dPathDependQuality);

  report(prb::test::indexAmortizingSwap, dRate, dInterval, iPoints, uModel);
}

void prb::test::volatilityLinearInterpOfVar() 
{
  cfl::test::print("VOLATILITY CURVE OBTAINED BY THE LINEAR INTERPOLATION OF VARIANCE CURVE");
  cout << endl;
  double dSigma = 0.2;
  double dLambda = 0.1;
  double dInitialTime = 1.;
  unsigned iTimes = 10;
  std::vector<double> uTimes(iTimes);
  double dPeriod = 0.5;
  uTimes.front() = dInitialTime + dPeriod;
  std::transform(uTimes.begin(), uTimes.end()-1, uTimes.begin()+1, 
		 [dPeriod](double dX){ return dX+dPeriod; });
  std::vector<double> uVolatility(iTimes);
  cfl::Function uVolatilityCurve = 
    cfl::Data::volatility(dSigma, dLambda, dInitialTime);
  std::transform(uTimes.begin(), uTimes.end(), uVolatility.begin(), 
		 uVolatilityCurve); 

  cout << "initial time = " << dInitialTime << endl;
  cout << "input volatility times: " << endl;
  cfl::test::print(uTimes.begin(), uTimes.end());
  cout << "input volatilites: " << endl;
  cfl::test::print(uVolatility.begin(), uVolatility.end());
  
  Function uResult = 
    prb::volatilityLinearInterpOfVar(uTimes, uVolatility, 
				     dInitialTime);
  double dInterval = 4.25;
  cout << "output volatilites: " << endl;
  cfl::test::Data::print(uResult, dInitialTime, dInterval); 
}

MultiFunction prb::test::
compoundCallAmericanPut(AssetModel & rModel) 
{
  cfl::test::print("COMPOUND CALL OPTION ON AMERICAN PUT IN BLACK MODEL");
  cout << endl;
  double dMaturity = 0.5*(rModel.initialTime() 
			  + cfl::test::Black::c_uExerciseTimes.front());
  double dCallStrike = 0.1* cfl::test::Black::c_dSpot;
  double dPutStrike = cfl::test::Black::c_dSpot; 
  std::vector<double> uPutExerciseTimes = cfl::test::Black::c_uExerciseTimes; 
  cout << "maturity for compound call = " << dMaturity << endl;
  cout << "call strike = " << dCallStrike << endl;
  cout << "put strike = " << dPutStrike << endl;
  cout << "exercise times = "<< endl;
  cfl::test::print(uPutExerciseTimes.begin(), uPutExerciseTimes.end());

  return prb::compoundCallAmericanPut(dMaturity, dCallStrike,  
				      dPutStrike, uPutExerciseTimes, rModel);
}

cfl::MultiFunction prb::test::
callableCappedFloater(InterestRateModel & rModel)
{
  cfl::test::print("CALLABLE CAPPED FLOATER IN INTEREST RATE MODEL");
  cout << endl;
  cfl::Data::CashFlow uCapParameters = cfl::test::HullWhite::swapParameters();
  uCapParameters.rate = cfl::test::HullWhite::c_dCapRate;
  cout << "cap parameters:" << endl;
  cfl::test::print(uCapParameters);
  double dLiborSpread = 0.01;
  cout << "spread over LIBOR = " << dLiborSpread << endl;

  return prb::callableCappedFloater(uCapParameters, dLiborSpread, rModel);
}

double amortization(double dRate) 
{
  return 1. - ::exp(-5.*dRate);
}

cfl::MultiFunction prb::test::
indexAmortizingSwap(InterestRateModel & rModel, bool bPayFloat)
{
 
  cfl::test::print("INDEX AMORTIZING SWAP IN INTEREST RATE MODEL");
  cout << endl;
  cfl::Data::Swap uSwapParameters = cfl::test::HullWhite::swapParameters();
  double dCleanUpThreshold = 0.5; 
  uSwapParameters.payFloat = bPayFloat; 
  cout << "swap parameters:" << endl;
  cfl::test::print(uSwapParameters);
  cout << "amortizing function for notional is f(x) =  1 - exp(-5x)" << endl;
  cout << "clean up threshold = " << dCleanUpThreshold << endl;

  return  prb::indexAmortizingSwap(uSwapParameters, amortization, 
				   dCleanUpThreshold, rModel);
}
